"""
Trajectopy - Trajectory Evaluation in Python

Gereon Tombrink, 2023
mail@gtombrink.de
"""
import datetime
from enum import Enum, auto
import numpy as np


class Unit(Enum):
    """
    Enumeration class handling different units
    """

    DEGREE = auto()
    RADIAN = auto()
    METER = auto()
    SECOND = auto()
    SCALE = auto()
    NONE = auto()


# [multiply with , unit string, number of decimals]
UNIT_FORMAT_RULES = {
    Unit.METER: {"normal": [1, "m", 4], "precise": [1000, "mm", 1]},
    Unit.RADIAN: {"normal": [180 / np.pi, "°", 4]},
    Unit.DEGREE: {"normal": [1, "°", 4]},
    Unit.SECOND: {"normal": [1000, "ms", 1], "precise": [1000, "ms", 1]},
    Unit.SCALE: {"normal": [1, "-", 4], "precise": [1e6, "ppm", 1]},
    Unit.NONE: {"normal": [1, "-", 4]},
}


class RotApprox(Enum):
    """
    Enumeration class handling different techniques for rpy approximation
    """

    WINDOW = 0
    INTERP = 1
    CUBIC = 2

    def __str__(self) -> str:
        if self.value == 0:
            return "window"

        if self.value == 1:
            return "lap_interp"

        if self.value == 2:
            return "cubic"


GPS_LEAP_SECONDS = 18
GPS_WEEK_ZERO = datetime.datetime(1980, 1, 6, 0, 0, 0)


class TimeFormat(Enum):
    UNIX = auto()
    DATETIME = auto()
    GPS_SOW = auto()


TIME_FORMAT_DICT = {"unix": TimeFormat.UNIX, "datetime": TimeFormat.DATETIME, "gps_sow": TimeFormat.GPS_SOW}
